#!/usr/local/bin/python
#
# $Id: getnews.py,v 1.26 2002/03/16 16:22:46 dhellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Import RDF files as CMF News Objects

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: getnews.py,v $',
    'rcs_id'       : '$Id: getnews.py,v 1.26 2002/03/16 16:22:46 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <DougHellmann@bigfoot.com>',
    'project'      : 'UNSPECIFIED',
    'created'      : 'Sat, 10-Nov-2001 06:22:54 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.26 $',
    'date'         : '$Date: 2002/03/16 16:22:46 $',
}

#
# Import system modules
#
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO
import urllib2
from xml.sax import saxlib, saxexts
import xml.parsers.expat

#
# Import Local modules
#
import slashbox

#
# This must be imported last because it changes the cwd and
# breaks imports of some of the other modules.
#
import CMFFeedApp

#
# Module
#

class GetNews(CMFFeedApp.CMFFeedApp):

    config_file_base = 'getnews.conf'

    examplesDescription = """
    To test a specific feed::

        % ./getnews.py --test FolderName
        
    """
    
    default_config_values = {}
    default_config_values.update(CMFFeedApp.CMFFeedApp.default_config_values)
    default_config_values.update({
        'portal_name':'hellfly.Portal',
        'fixup_translation_table':[],
        })

    builtin_fixup_translation_table = [
        # Some feeds fail to escape & characters with
        # an entity reference.  Lazy?
        ('& ', '&amp;'),
        ('Q&A', 'Q&amp;A'),
        ('&Co', '&amp; Co'),
        
        # Apparently registration marks aren't valid
        # in XML?  The parser croaks.
        ('&reg;', '(R)'),
        
        # Embedded null values?  What gives with *that*?
        (chr(0x00), ''),
        
        ]

    debugging = 0
    def optionHandler_debug(self):
        "Turn on debug mode to dump exceptions."
        self.debugging = 1
        return

    test_folder = None
    def optionHandler_test(self, testFolderName):
        "Turn on test mode, and only process testFolderName."
        self.test_mode = 1
        self.test_folder = testFolderName
        return

    def fixupContents(self, textBody):
        new_body = textBody

        #
        # First the user overrides
        #
        for search_for, replace_with in self.fixup_translation_table:
            self.statusMessage('  Translating "%s"=>"%s"' % (search_for,
                                                           replace_with),
                               3)
            new_body = new_body.replace(search_for, replace_with)
            
        #
        # Then the builtins
        #
        for search_for, replace_with in self.builtin_fixup_translation_table:
            self.statusMessage('  Translating "%s"=>"%s"' % (search_for,
                                                           replace_with),
                               3)
            new_body = new_body.replace(search_for, replace_with)
        return new_body
    
    def getStoriesFromRDFFile(self, rdfUrl):
        self.statusMessage('Retrieving %s...' % rdfUrl)
        contents = urllib2.urlopen(rdfUrl)

        body=contents.read()
        
        if self.test_mode:
            self.statusMessage(body, 2)
            contents = StringIO(body)

        self.statusMessage('  Fixing up content stream', 2)
        new_body = self.fixupContents(body)

        new_stream = StringIO(new_body)

        self.statusMessage('  Parsing', 2)
        doc_handler = slashbox._myrdf()
        xml_parser = saxexts.make_parser()
        xml_parser.setDocumentHandler(doc_handler)
        try:
            xml_parser.parseFile(new_stream)
        except xml.parsers.expat.ExpatError, msg:
            if self.debugging:
                print new_body
            else:
                self.statusMessage('BAD CONTENT:', 3)
                self.statusMessage(new_body, 3)
            raise
        
        l = doc_handler.stories()
        l.reverse()
        return l

    def storyToId(self, story):
        url = story['link']
        return self.urlToId(url)
    
    
    def loadDataForFolder(self, folder):
        """Find the RDF file(s) for the folder.

        Read the RDF data, and store the results.
        """
        #
        # Verify that we want to process this file.
        #
        if self.test_mode:
            if folder.id != self.test_folder:
                self.statusMessage('Skipping %s' % folder.id)
                return
        #
        # Get the URL for the feed
        #
        try:
            rdf = folder.RDF
        except AttributeError:
            self.statusMessage(
                'Folder %s has no RDF object, skipping.' % folder.id
                )
            return
        
        try:
            url = rdf.getRemoteUrl()
        except:
            self.errorMessage('Could not get remote URL for %s' % folder.id)
            return
        if not url:
            self.errorMessage('Remote URL for %s is empty, skipping' % \
                              folder.id)
            return
        #
        # Get the stories from the feed
        #
        try:
            stories = self.getStoriesFromRDFFile(url)
        except urllib2.URLError, msg:
            self.errorMessage('Could not load %s (%s)' % (url, msg))
        except:
            import sys
            self.errorMessage('Could not load %s (%s)' % (url,
                                                          sys.exc_value))
            if self.debugging:
                raise
        else:
            #
            # Store the stories from the feed
            #
            for story in stories:
                id = self.storyToId(story)
                self.addNewsItem(folder, id, story['title'], story['link'],
                                 story['description'],
                                 subject=folder.title_or_id())
        return

if __name__ == '__main__':
    try:
        GetNews().run()
    except GetNews.HelpRequested:
        pass
    
